// $Id: clock.js,v 1.8 2010/01/28 14:24:36 tstoeckler Exp $

/*
 * Gets the date format and time zone related parameters from PHP and formats the date according
 * to the php date formatters. Visit http://php.net/date for more information. Using jQuery Timers (http://plugins.jquery.com/project/timers) the
 * date is recalculated every second to make the clock dynamic.
 */
Drupal.behaviors.clockDisplay = function (context) {
  // Gets a PHP timestamp, which counts the seconds since the begin of the UNIX-era.
  var clockTimestampString = Drupal.settings['0'];
  // The time zone offset in seconds.
  var clockTimezoneOffsetSeconds = Drupal.settings['1'];
  // The name of the timezone, e.g. 'Europe/London'.
  var clockTimezoneName = Drupal.settings['2'];
  // Daylight Savings Time information. '1' for yes, '0' for no.
  var clockDaylightSavingsTime = Drupal.settings['3'];
  // The name of the offset, e.g. 'GMT'.
  var clockOffsetName = Drupal.settings['4'];
  // A PHP date format string.
  var clockDateFormat = Drupal.settings['5'];

  // Calculate the timestamp with the time zone offset and turn it into milliseconds.
  // JavaScript's time handling is based on milliseconds, while PHP's is based on seconds.
  // TODO: Either remove the "-3600"-hack that currently makes this work, or figure out why it is needed.
  var clockTimestamp = (parseInt(clockTimestampString) - 3600 + parseInt(clockTimezoneOffsetSeconds)) * 1000;

  // Calculate whether it is a leap year or not. Years that are multiples of 4 are leap years,
  // while multiples of 100 are not, but multiples of 400 are. That way 1600, 2000, 2004, 2008 were leap years, while 1700, 1800, 1900 were not.
  var clockDate = new Date(clockTimestamp);
  var year = clockDate.getFullYear();
  if (((year % 4) == 0) && ((year % 100) != 0) || ((year % 400) == 0)) {
    var leapYear = '1';
  }
  else {
    var leapYear = '0';
  }

  // Create an array containing month names and lengths.
  var months = new Array();
  months[1] = new Object();
  months[1]['longName'] = Drupal.t('January');
  months[1]['shortName'] = Drupal.t('Jan');
  months[1]['days'] = 31;
  months[2] = new Object();
  months[2]['longName'] = Drupal.t('February');
  months[2]['shortName'] = Drupal.t('Feb');
  months[2]['days'] = ((leapYear == 1) ? 29 : 28);
  months[3] = new Object();
  months[3]['longName'] = Drupal.t('March');
  months[3]['shortName'] = Drupal.t('Mar');
  months[3]['days'] = 31;
  months[4] = new Object();
  months[4]['longName'] = Drupal.t('April');
  months[4]['shortName'] = Drupal.t('Apr');
  months[4]['days'] = 30;
  months[5] = new Object();
  months[5]['longName'] = Drupal.t('May');
  months[5]['shortName'] = Drupal.t('May');
  months[5]['days'] = 31;
  months[6] = new Object();
  months[6]['longName'] = Drupal.t('June');
  months[6]['shortName'] = Drupal.t('Jun');
  months[6]['days'] = 30;
  months[7] = new Object();
  months[7]['longName'] = Drupal.t('July');
  months[7]['shortName'] = Drupal.t('Jul');
  months[7]['days'] = 31;
  months[8] = new Object();
  months[8]['longName'] = Drupal.t('August');
  months[8]['shortName'] = Drupal.t('Aug');
  months[8]['days'] = 31;
  months[9] = new Object();
  months[9]['longName'] = Drupal.t('September');
  months[9]['shortName'] = Drupal.t('Sep');
  months[9]['days'] = 30;
  months[10] = new Object();
  months[10]['longName'] = Drupal.t('October');
  months[10]['shortName'] = Drupal.t('Oct');
  months[10]['days'] = 31;
  months[11] = new Object();
  months[11]['longName'] = Drupal.t('November');
  months[11]['shortName'] = Drupal.t('Nov');
  months[11]['days'] = 30;
  months[12] = new Object();
  months[12]['longName'] = Drupal.t('December');
  months[12]['shortName'] = Drupal.t('Dec');
  months[12]['days'] = 31;

  // Create an array containing weekday names.
  var weekdays = new Array();
  weekdays[0] = new Object();
  weekdays[0]['longName'] = Drupal.t('Sunday');
  weekdays[0]['shortName'] = Drupal.t('Sun');
  weekdays[1] = new Object();
  weekdays[1]['longName'] = Drupal.t('Monday');
  weekdays[1]['shortName'] = Drupal.t('Mon');
  weekdays[2] = new Object();
  weekdays[2]['longName'] = Drupal.t('Tuesday');
  weekdays[2]['shortName'] = Drupal.t('Tue');
  weekdays[3] = new Object();
  weekdays[3]['longName'] = Drupal.t('Wednesday');
  weekdays[3]['shortName'] = Drupal.t('Wed');
  weekdays[4] = new Object();
  weekdays[4]['longName'] = Drupal.t('Thursday');
  weekdays[4]['shortName'] = Drupal.t('Thu');
  weekdays[5] = new Object();
  weekdays[5]['longName'] = Drupal.t('Friday');
  weekdays[5]['shortName'] = Drupal.t('Fri');
  weekdays[6] = new Object();
  weekdays[6]['longName'] = Drupal.t('Saturday');
  weekdays[6]['shortName'] = Drupal.t('Sat');


  // Initialize the clock variable for later use.
  var clock = '';

  // Format a given date format string into a proper date string.
  function formatDate(date, timestamp, dateFormat, timezoneOffsetSeconds, timezoneName, daylightSavingsTime, offsetName) {
    // Initialize the 'formattedDate' variable for later use.
    var formattedDate = '';

    // Prepare variables for the format conversion.

    // Year-related.
    var year = date.getFullYear();
    var yearShort = year % 100;

    // Month-related.
    var month = date.getMonth();
    // JavaScript starts counting the months with 0 instead of 1.
    month++;
    var monthLeadingZero = ((month < 10) ? "0" + month : month);
    var monthLongName = months[month]['longName'];
    var monthShortName = months[month]['shortName'];

    var day = date.getDate();
    var dayLeadingZero = ((day < 10) ? "0" + day : day);
    switch (day) {
      case "1":
        var dayOfMonthAppend = Drupal.t('st');
        break;
      case "2":
        var dayOfMonthAppend = Drupal.t('nd');
        break;
      case "3":
        var dayOfMonthAppend = Drupal.t('rd');
      default:
        var dayOfMonthAppend = Drupal.t('th');
        break;
    }
    // To calculate the days of the year, iterate through all passed months and then add the current days of the month.
    var daysOfYear = '';
    for (var m = 1; m < month; m++) {
      daysOfYear += months[m]['days'];
    }
    daysOfYear += day;

    // Week-related.
    var weekday = date.getDay();
    if (weekday == 0) {
      var weekNumber = Math.floor((daysOfYear - 7) / 7) + 1;
    }
    else {
      var weekNumber = Math.floor((daysOfYear - weekday) / 7) + 1;
    }
    var weekNumberLeadingZero = ((weekNumber < 10) ? '0' + weekNumber : weekNumber);
    var weekdayLongName = weekdays[weekday]['longName'];
    var weekdayShortName = weekdays[weekday]['shortName'];

    // Offset-related.
    if (timezoneOffsetSeconds < 0) {
      var offsetPrefix = '-';
      var offsetHours = Math.floor(parseInt(timezoneOffsetSeconds.substr(1)) / 3600);
      var offsetHoursLeadingZero = ((offsetHours < 10) ? '0' + offsetHours : offsetHours);
      var offsetMinutes = (parseInt(timezoneOffsetSeconds.substr(1)) / 60) % 60;
      var offsetMinutesLeadingZero = ((offsetMinutes < 10) ? '0' + offsetMinutes : offsetMinutes);
    }
    else {
      var offsetPrefix = '+';
      var offsetHours = Math.floor(timezoneOffsetSeconds) / 3600;
      var offsetHoursLeadingZero = ((offsetHours < 10) ? '0' + offsetHours : offsetHours);
      var offsetMinutes = (timezoneOffsetSeconds / 60) % 60;
      var offsetMinutesLeadingZero = ((offsetMinutes < 10) ? '0' + offsetMinutes : offsetMinutes);
    }

    // Hour-related.
    var hours = date.getHours();
    var hoursLeadingZero = ((hours < 10) ? "0" + hours : hours);
    if (hours == 0) {
      var hoursTwelve = (hours + 12);
    }
    else if (hours > 12) {
      var hoursTwelve = (hours - 12);
    }
    else {
      var hoursTwelve = hours;
    }
    var hoursTwelveLeadingZero = ((hoursTwelve < 10) ? '0' + hoursTwelve : hoursTwelve);

    // Minute-related.
    var minutes = date.getMinutes();
    var minutesLeadingZero = ((minutes < 10) ? "0" + minutes : minutes);

    // Second-related.
    var seconds = date.getSeconds();
    var secondsLeadingZero = ((seconds < 10) ? "0" + seconds : seconds);

    // Turn the date format string into an array so each character has its own key.
    var dateFormatParts = dateFormat.length;
    var dateFormatArray = dateFormat.split("");

    // Perform the date format conversion for a character.
    for (i=0; i < dateFormatParts; i++) {
      switch (dateFormatArray[i]) {
        // If the escape character "\" is used, simply return the following character.
        case "\\":
          formattedDate += dateFormatArray[++i];
          break;
        // 'am' or 'pm' depending on the time of day.
        case "a":
          formattedDate += ((hours >= 12) ? Drupal.t('pm') : Drupal.t('am'));
          break;
        // 'AM' or 'PM' depending on the time of day.
        case "A":
          formattedDate += ((hours >= 12) ? Drupal.t('PM') : Drupal.t('AM'));
          break;
        // Swatch-Internet-Time.
        case "B":
          formattedDate += Math.floor(((hours * 3600) + (minutes * 60) + seconds - parseInt(clockTimezoneSeconds) + 3600) / 86400 * 1000);
          break;
        // ISO-8601 date.
        case "c":
          formattedDate += year + "-" + monthLeadingZero + "-" + dayLeadingZero + "T" + hoursLeadingZero + ":" + minutesLeadingZero + ":" + secondsLeadingZero + offsetPrefix + offsetHoursLeadingZero + ":" + offsetMinutesLeadingZero;
          break;
        // Day of month with leading zero, '01' - '31'.
        case "d":
          formattedDate += dayLeadingZero;
          break;
        // Time zone name, e.g. 'Europe/London';
        case "e":
          formattedDate += timezoneName;
          break;
        // Short name of weekday, e.g. 'Sun'.
        case "D":
          formattedDate += weekdayShortName;
          break;
        // Name of month, e.g. 'January'.
        case "F":
          formattedDate += monthLongName;
          break;
        // Hours in 12-hour format, '1' - '12'.
        case "g":
          formattedDate += hoursTwelve;
          break;
        // Hours in 24-hour format, '0' - '23'.
        case "G":
          formattedDate += hours;
          break;
        // Hours in 12-hour format with leading zero, '01' - '12'.
        case "h":
          formattedDate += hoursTwelveLeadingZero;
          break;
        // Hours in 24-hour format with leading zero, '00' - '23'.
        case "H":
          formattedDate += hoursLeadingZero;
          break;
        // Minutes with leading zero, '00' - '59'.
        case "i":
          formattedDate += minutesLeadingZero;
          break;
        // Daylight Savings Time, '1' or '0'
        case "I":
          formattedDate += daylightSavingsTime;
          break;
        // Day of month, '1' - '31'
        case "j":
          formattedDate += day;
          break;
        // Name of weekday, e.g. 'Sunday'.
        case "l":
          formattedDate += weekdayLongName;
          break;
        // Leap year, '1' or '0'.
        case "L":
          formattedDate += leapYear;
          break;
        // Number of month with leading zero, '01' - '12'
        case "m":
          formattedDate += monthLeadingZero;
          break;
        // Short name of month, e.g. 'Jan'.
        case "M":
          formattedDate += monthShortName;
          break;
        // Number of month, '1' - '12'.
        case "n":
          formattedDate += month;
          break;
        // Time zone offset, e.g. '+1030'
        case "O":
          formattedDate += offsetPrefix + offsetHours + offsetMinutes;
          break;
        // Time zone offset, e.g. '+10:30'.
        case "P":
          formattedDate += offsetPrefix + offsetHours + ":" + offsetMinutes;
          break;
        // RFC-2822 date.
        case "r":
          formattedDate += weekdayShortName + ", " + dayLeadingZero + " " + monthShortName + " " + year + " " + hoursLeadingZero + ":" + minutesLeadingZero + ":" + secondsLeadingZero + " " + clockPHPDateO;
          break;
        // Seconds with leading zero.
        case "s":
          formattedDate += secondsLeadingZero;
          break;
        // Appendage for the day of month, e.g. 'st' if the day is '1'.
        case "S":
          formattedDate += dayOfMonthAppend;
          break;
        // Total days of month, '28' - '31'.
        case "t":
          formattedDate += months[month]['days'];
          break;
        // Name of offset, e.g. 'GMT'.
        case "T":
          formattedDate += offsetName;
          break;
        // Seconds since the begin of the UNIX-era.
        // Note that this outputs a PHP-timestamp not a JavaScript timestamp, which would be in milliseconds.
        case "U":
          formattedDate += timestamp;
          break;
        // Number of weekday, '0' for Sunday, ..., '6' for Saturday.
        case "w":
          formattedDate += weekday;
          break;
        // Week number with leading zero, '01' - '52'.
        case "W":
          formattedDate += weekNumberLeadingZero;
          break;
        // Year, e.g. '2001'.
        case "Y":
          formattedDate += year;
          break;
        // Short year, e.g. '01'.
        case "y":
          formattedDate += yearShort;
          break;
        // Days of the year, '0' - '365'.
        // Since PHP starts counting the days of the year at '0', we need subtract one.
        case "z":
          formattedDate += daysOfYear - 1;
          break;
        // Time zone offset in seconds.
        case "Z":
          formattedDate += timezoneOffsetSeconds;
          break;
        // If the character is not a date formatter, just add it to the output.
        default:
          formattedDate += dateFormatArray[i];
          break;
      }
    }
    return formattedDate;
  }


  // Recalculate the date every second. ".everyTime" is provided by jQuery Timers.
  $(document).everyTime(1000, function() {
    // Add 1 second to the timestamp.
    clockTimestamp = clockTimestamp + 1000;
    // Format the timestamp into the given format.
    clockDate = new Date(clockTimestamp);
    clock = formatDate(clockDate, clockTimestamp, clockDateFormat, clockTimezoneOffsetSeconds, clockTimezoneName, clockDaylightSavingsTime, clockOffsetName);
    $("div.clock").html(clock);
  }, 0);

}

/*
 * Hide the select boxes on the clock block configure screen if the specific options are disabled.
 */
Drupal.behaviors.clockConfigure = function (context) {
  // Show the custom time zone select list only when 'Custom time zone' is checked.
  $("div.edit-custom-timezone").addClass("js-hide");
  function customTimezoneSelect() {
    var value = $("div.edit-timezone input:checked").val();
    if (value == '3') {
      $("div.edit-custom-timezone").show();
    }
    else {
      $("div.edit-custom-timezone").hide();
    }
  }
  $("div.edit-timezone").bind("click", customTimezoneSelect);
  customTimezoneSelect();

  // Show the custom date format select list only when 'Custom date format' is checked.
  $("div.edit-custom-date-format").addClass("js-hide");
  function customDateFormatSelect() {
    var value = $("div.edit-date-format input:checked").val();
    if (value == 'custom') {
      $("div.edit-custom-date-format").show();
    }
    else {
      $("div.edit-custom-date-format").hide();
    }
  }
  $("div.edit-date-format").bind("click", customDateFormatSelect);
  customDateFormatSelect();
}

